function toggleMode(toggle, id) {
  gantt.$zoomToFit = !gantt.$zoomToFit;
  if (gantt.$zoomToFit) {
    toggle.innerHTML = "Set default Scale";
    //Saving previous scale state for future restore
    saveConfig();
    zoomToFit(id);
  }
  else {

    toggle.innerHTML = "Zoom to Fit";
    //Restore previous scale state
    restoreConfig();
    gantt.render();
  }
}

var cachedSettings = {};

function saveConfig() {
  var config = gantt.config;
  cachedSettings = {};
  cachedSettings.scales = config.scales;
  cachedSettings.start_date = config.start_date;
  cachedSettings.end_date = config.end_date;
  cachedSettings.scroll_position = gantt.getScrollState();
}

function restoreConfig() {
  applyConfig(cachedSettings);
}

function applyConfig(config, dates, id = 0) {
  if (Gantt[id] !== undefined) {
    Gantt[id].config.scales = config.scales;
  }
  else {
    gantt.config.scales = config.scales;
  }

  // restore the previous scroll position
  if (config.scroll_position) {
    setTimeout(function () {
      gantt.scrollTo(config.scroll_position.x, config.scroll_position.y)
    }, 4)
  }
}


function zoomToFit(id = 0) {
  let project = gantt.getSubtaskDates(),
    areaWidth = 300;
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    project = Gantt[id].getSubtaskDates(),
      areaWidth = Gantt[id].$task.offsetWidth;
  }
  else {
    areaWidth = gantt.$task.offsetWidth;
  }
  let scaleConfigs = zoomConfig.levels;
  for (var i = 0; i < scaleConfigs.length; i++) {
    var columnCount = getUnitsBetween(project.start_date, project.end_date, scaleConfigs[i].scales[scaleConfigs[i].scales.length - 1].unit, scaleConfigs[i].scales[0].step);
    if ((columnCount + 2) * gantt.config.min_column_width <= areaWidth) {
      break;
    }
  }
  if (i == scaleConfigs.length) {
    i--;
  }

  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].ext.zoom.setLevel(scaleConfigs[i].name);
  }
  else {
    gantt.ext.zoom.setLevel(scaleConfigs[i].name);
  }
  applyConfig(scaleConfigs[i], project, id);
}

// Get number of columns in timeline.
function getUnitsBetween(from, to, unit, step) {
  var start = new Date(from),
    end = new Date(to);
  var units = 0;
  while (start.valueOf() < end.valueOf()) {
    units++;
    start = gantt.date.add(start, step, unit);
  }
  return units;
}

function zoom_in(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].ext.zoom.zoomOut();
    Gantt[id].$zoomToFit = false;
  }
  else {
    gantt.ext.zoom.zoomOut();
    gantt.$zoomToFit = false;
  }
  jQuery(".zoomToFit").html("<i class='bi bi-fullscreen-exit'></i> Zoom to Fit");
}

function zoom_out(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].ext.zoom.zoomIn();
    Gantt[id].$zoomToFit = false;
  }
  else {
    gantt.ext.zoom.zoomIn();
    gantt.$zoomToFit = false;
  }
  jQuery(".zoomToFit").html("<i class='bi bi-arrows-fullscreen'></i> Zoom to Fit");
}

function collapse(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].batchUpdate(function () {
      Gantt[id].eachTask(function (task) {
        Gantt[id].close(task.id)
      });
    });
  }
  else {
    gantt.batchUpdate(function () {
      gantt.eachTask(function (task) {
        gantt.close(task.id)
      });
    });
  }
}

function expand(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].batchUpdate(function () {
      Gantt[id].eachTask(function (task) {
        Gantt[id].open(task.id)
      });
    });
  }
  else {
    gantt.batchUpdate(function () {
      gantt.eachTask(function (task) {
        gantt.open(task.id)
      });
    });
  }
}

function undo(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].undo();
  }
  else {
    gantt.undo();
  }
}

function redo(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].redo();
  }
  else {
    gantt.redo();
  }
}

function exportToExcel(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].exportToExcel();
  }
  else {
    gantt.exportToExcel();
  }
}

function exportToMSProject(id, config) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].exportToMSProject(config);
  }
  else {
    gantt.exportToMSProject(config);
  }
}

function exportToICal(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].exportToICal();
  }
  else {
    gantt.exportToICal();
  }
}

function fullscreen(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].ext.fullscreen.toggle();
  }
  else {
    gantt.ext.fullscreen.toggle();
  }
}

var zoomConfig = {
  levels: [
    // Hours.
    {
      name: "hour",
      scale_height: 27,
      scales: [
        {unit: "day", step: 1, format: "%d %M"},
        {unit: "hour", step: 1, format: "%H:%i"},
      ]
    },
    // Days.
    {
      name: "day",
      scale_height: 27,
      scales: [
        {unit: "day", step: 1, format: "%d %M"}
      ]
    },
    // Weeks.
    {
      name: "week",
      scale_height: 50,
      scales: [
        {
          unit: "week", step: 1, format: function (date) {
            var dateToStr = gantt.date.date_to_str("%d %M");
            var endDate = gantt.date.add(date, -6, "day");
            var weekNum = gantt.date.date_to_str("%W")(date);
            return "#" + weekNum + ", " + dateToStr(date) + " - " + dateToStr(endDate);
          }
        },
        {unit: "day", step: 1, format: "%j %D"}
      ]
    },
    // Months.
    {
      name: "month",
      scale_height: 50,
      scales: [
        {unit: "month", step: 1, format: "%F, %Y"},
        {
          unit: "week", step: 1, format: function (date) {
            var dateToStr = gantt.date.date_to_str("%d %M");
            var endDate = gantt.date.add(gantt.date.add(date, 1, "week"), -1, "day");
            return dateToStr(date) + " - " + dateToStr(endDate);
          }
        }
      ]
    },
    // Quarters.
    {
      name: "quarter",
      height: 50,
      scales: [
        {
          unit: "quarter", step: 3, format: function (date) {
            var dateToStr = gantt.date.date_to_str("%M %y");
            var endDate = gantt.date.add(gantt.date.add(date, 3, "month"), -1, "day");
            return dateToStr(date) + " - " + dateToStr(endDate);
          }
        },
        {unit: "month", step: 1, format: "%M"},
      ]
    },
    // Years.
    {
      name: "year",
      scale_height: 50,
      scales: [
        {
          unit: "year", step: 5, format: function (date) {
            var dateToStr = gantt.date.date_to_str("%Y");
            var endDate = gantt.date.add(gantt.date.add(date, 5, "year"), -1, "day");
            return dateToStr(date) + " - " + dateToStr(endDate);
          }
        }
      ]
    },
    // Decades.
    {
      name: "year",
      scale_height: 50,
      scales: [
        {
          unit: "year", step: 100, format: function (date) {
            var dateToStr = gantt.date.date_to_str("%Y");
            var endDate = gantt.date.add(gantt.date.add(date, 100, "year"), -1, "day");
            return dateToStr(date) + " - " + dateToStr(endDate);
          }
        },
        {
          unit: "year", step: 10, format: function (date) {
            var dateToStr = gantt.date.date_to_str("%Y");
            var endDate = gantt.date.add(gantt.date.add(date, 10, "year"), -1, "day");
            return dateToStr(date) + " - " + dateToStr(endDate);
          }
        },
      ]
    },
  ],
  element: function () {
    return gantt.$root.querySelector(".gantt_task");
  }
};

var call_lag_Gantt = {};
var call_lag_gantt = null;

function auto_schedule(e, id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    if (!call_lag_Gantt[id]) {
      linkLagEditor(Gantt[id], id);
    }
    Gantt[id].config.auto_scheduling = !Gantt[id].config.auto_scheduling;
    if (Gantt[id].config.auto_scheduling) {
      e.classList.add("btn-primary");
    }
    else {
      e.classList.remove("btn-primary");
    }
  }
  else {
    if (!call_lag_gantt) {
      linkLagEditor(gantt);
    }
    gantt.config.auto_scheduling = !gantt.config.auto_scheduling;
    if (gantt.config.auto_scheduling) {
      e.classList.add("btn-primary");
    }
    else {
      e.classList.remove("btn-primary");
    }
  }
}

function linkLagEditor(gantt, id = null) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    call_lag_Gantt[id] = true;
  }
  else {
    call_lag_gantt = true;
  }
  function endPopup() {
    modal = null;
    editLinkId = null;
  }

  function cancelEditLink() {
    endPopup()
  }

  function deleteLink() {
    gantt.deleteLink(editLinkId);
    endPopup()
  }

  function saveLink() {
    let link = gantt.getLink(editLinkId);

    let lagValue = modal.querySelector(".lag-input").value;
    if (!isNaN(parseInt(lagValue, 10))) {
      link.lag = parseInt(lagValue, 10);
    }

    gantt.updateLink(link.id);
    if (gantt.autoSchedule) {
      gantt.autoSchedule(link.source);
    }
    endPopup();
  }

  let modal;
  let editLinkId;
  gantt.attachEvent("onLinkDblClick", function (id, e) {
    editLinkId = id;
    let link = gantt.getLink(id);
    let linkTitle;
    switch (link.type) {
      case gantt.config.links.finish_to_start:
        linkTitle = "FS";
        break;
      case gantt.config.links.finish_to_finish:
        linkTitle = "FF";
        break;
      case gantt.config.links.start_to_start:
        linkTitle = "SS";
        break;
      case gantt.config.links.start_to_finish:
        linkTitle = "SF";
        break;
    }

    linkTitle += " <b>" + gantt.getTask(link.source).text + "</b> link to <b>" + gantt.getTask(link.target).text + "</b>";

    if (gantt.config.auto_scheduling) {
      modal = gantt.modalbox({
        title: "<div class='fs-5'>" + linkTitle + "</div>",
        text: "<div>" +
          "<label>Lag <input type='number' class='lag-input' /></label>" +
          "</div>",
        buttons: [
          {label: "Save", css: "link-save-btn", value: "save"},
          {label: "Cancel", css: "link-cancel-btn", value: "cancel"},
          {label: "Delete", css: "link-delete-btn", value: "delete"}
        ],
        width: "500px",
        type: "popup-css-class-here",
        callback: function (result) {
          switch (result) {
            case "save":
              saveLink();
              break;
            case "cancel":
              cancelEditLink();
              break;
            case "delete":
              deleteLink();
              break;
          }
        }
      });
      modal.querySelector(".lag-input").value = link.lag || 0;
    }
    else {
      modal = gantt.confirm({
        text: Drupal.t("Link <b>" + gantt.getTask(link.source).text + "</b> - <b>" + gantt.getTask(link.target).text + "</b> will be deleted"),
        ok: Drupal.t("OK"),
        cancel: Drupal.t("Cancel"),
        callback: function (result) {
          if (result) {
            deleteLink();
          }
          else {
            cancelEditLink();
          }
        }
      });
    }
  });
  return true;
}

// Group server list.
function groupGantt(e, id, group_name) {
  let listResource = e.closest('.btn-group-resource').querySelectorAll('.dropdown-item');
  listResource.forEach((element) => {
    element.classList.remove('active');
  });
  e.classList.add("active");
  let listButton = e.closest('.btn-group-resource').querySelectorAll("button");
  if (e.dataset.field_resource === "default") {
    listButton.forEach((element) => {
      element.classList.remove('btn-primary');
    });
  }
  else {
    listButton.forEach((element) => {
      element.classList.add("btn-primary");
    });
  }
  e.closest('.btn-group-resource').firstElementChild.textContent = e.dataset.label;
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    showGroups(Gantt[id], group_name)
  }
  else {
    showGroups(gantt, group_name)
  }
}

function showGroups(gantt, group_name) {
  if (group_name !== "default") {
    gantt.groupBy({
      groups: gantt.serverList(group_name),
      relation_property: group_name,
      group_id: "key",
      group_text: "label"
    });
  }
  else {
    gantt.groupBy(false);
  }
}

// Search task name.
var filter_data = {};
function change_detector(e, id) {
  filter_data[id] = e.value;
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].refreshData();
  }
  else {
    gantt.refreshData();
  }
}

function compare_input(gantt, id_gantt, id = 0) {
  let match = false;
  // Check children's text.
  if (gantt.hasChild(id)) {
    gantt.eachTask(function (child_object) {
      if (compare_input(gantt, id_gantt, child_object.id, filter_data[id_gantt])) {
        match = true;
      }
    }, id);
  }

  // Check task's text.
  if(filter_data[id_gantt]) {
    if (gantt.getTask(id).text.toLowerCase().indexOf(filter_data[id_gantt].toLowerCase()) >= 0) {
      match = true;
    }
  }else {
    return true
  }

  return match;
}
